#include "irq_arm.h"
#include "rpc.h"

//---------------------------------------------------------------------------
ST_IRQ Irq;


//---------------------------------------------------------------------------
EWRAM_CODE void IrqInit(void)
{
	REG_IME = 0;

	_Memset(&Irq, 0x00, sizeof(ST_IRQ));

	INT_VECTOR    = (u32)IrqHandler;
	REG_IE        = IRQ_SERIAL | IRQ_VBLANK;
	REG_DISPSTAT  = LCDC_VBL;
	REG_SIOCNT    = SIO_IRQ | SIO_32BIT | SIO_START;
	REG_SIODATA32 = 0;

	REG_IME = 1;
}
//---------------------------------------------------------------------------
IWRAM_CODE void IrqHandler(void)
{
	REG_IME  = 0;
	u16 flag = REG_IF;

	if(flag & IRQ_SERIAL)
	{
		IrqHandlerSerial();
	}

	if(flag & IRQ_VBLANK)
	{
		IrqHandlerVblank();
	}

	REG_IF  = flag;
	REG_IME = 1;
}
//---------------------------------------------------------------------------
IWRAM_CODE void IrqHandlerSerial(void)
{
	REG_SIOCNT = SIO_IRQ | SIO_32BIT | SIO_START;

	if(Irq.isUse == FALSE)
	{
		return;
	}

	if(Irq.cntSend < Irq.cntStack)
	{
		REG_SIODATA32 = Irq.buf[Irq.cntSend++];

		return;
	}

	if(IRQ_SERIAL_CMD_END != Irq.buf[Irq.cntSend - 1])
	{
		// mbed̕ϐ̒li[܂
		*(u32*)(Irq.buf[Irq.cntSend - 1]) = REG_SIODATA32;
	}

	REG_SIODATA32 = 0;

	Irq.isUse = FALSE;
	Irq.cntSend = 0;
	Irq.cntStack = 0;
}
//---------------------------------------------------------------------------
IWRAM_CODE void IrqHandlerVblank(void)
{
	// EMPTY

	REG_IRQ_WAITFLAGS |= IRQ_VBLANK;
}
//---------------------------------------------------------------------------
IWRAM_CODE u32 IrqSerialGetData(void)
{
	return REG_SIODATA32;
}
//---------------------------------------------------------------------------
IWRAM_CODE void IrqSerialAdd(u32 d)
{
	Irq.buf[Irq.cntStack++] = d;
}
//---------------------------------------------------------------------------
IWRAM_CODE void IrqSerialAddCmdStart(void)
{
	IrqSerialAdd(IRQ_SERIAL_CMD_START);
}
//---------------------------------------------------------------------------
IWRAM_CODE void IrqSerialAddCmdEnd(void)
{
	IrqSerialAdd(IRQ_SERIAL_CMD_END);
}
//---------------------------------------------------------------------------
IWRAM_CODE void IrqSerialReady(void)
{
	Irq.isUse = TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE bool IrqSerialIsUse(void)
{
	return Irq.isUse;
}
